/*
 *  {\fvO
 *
 */

#include "integer.h"
#include "cpu.h"
#ifdef USE_SCIF
#include "scif.h"
#include "monitor.h"
#endif
#include "LCDpanel.h"
#include "graphics.h"
#include "grphcon.h"
#include "event.h"

#define	PCLK	24000000UL

void SetImask (unsigned long);	/* Set mask level (asmfunc.s) */

extern void table_viewer();
extern void text_demo();

/*---------------------------------------------------------*/
/* [NGA                                            */
/*---------------------------------------------------------*/

static volatile UINT Timer;		/* ptH[}X^C} (1kHzZ) */

#define	EVQLEN	30

static volatile struct {
  int ev_top;
  int ev_tail;
  int queue[EVQLEN];
} ev_queue;

void ev_add(int event)
{
    ev_queue.queue[ev_queue.ev_tail] = event;
    ev_queue.ev_tail++;
    if (ev_queue.ev_tail == EVQLEN) ev_queue.ev_tail = 0;
}

int ev_get(int do_wait)
{
    int ev_val;

    while (ev_queue.ev_top == ev_queue.ev_tail) {
	if (!do_wait) return 0;
    }
    ev_val = ev_queue.queue[ev_queue.ev_top++];
    if (ev_queue.ev_top == EVQLEN) ev_queue.ev_top = 0;
    return ev_val;
}

int ev_check()
{
    return (ev_queue.ev_top != ev_queue.ev_tail);
}

void ev_wait(int wevent)
{
	int event;

	do {
		event = ev_get(1);
	} while (event != wevent);
}

#define	PH0_MASK	(1<<0)
#define	PH1_MASK	(1<<1)
#define	PC7_MASK	(1<<7)
#define	PC6_MASK	(1<<6)
#define	PC5_MASK	(1<<5)
#define	PC2_MASK	(1<<2)

void key_timerproc()
{
    static BYTE prev_scan, key_state;
    BYTE work, change;

    work = (*PCPR0 & (PC7_MASK|PC6_MASK|PC5_MASK|PC2_MASK)) | (*PHPR0 & (PH0_MASK|PH1_MASK));
    if (work == prev_scan) {
	change = key_state;
	key_state = work;	/* update key status */
	change = change ^ work;
	if (change) {
	    if ((change & PH0_MASK) && !(work & PH0_MASK)) {
		ev_add(EV_PH0_DOWN);
	    }
	    if ((change & PH1_MASK) && !(work & PH1_MASK)) {
		ev_add(EV_PH1_DOWN);
	    }
	    if ((change & PC2_MASK) && !(work & PC2_MASK)) {
		ev_add(EV_PC2_DOWN);
	    }
	    if ((change & PC5_MASK) && !(work & PC5_MASK)) {
		ev_add(EV_PC5_DOWN);
	    }
	    if ((change & PC6_MASK) && !(work & PC6_MASK)) {
		ev_add(EV_PC6_DOWN);
	    }
	    if ((change & PC7_MASK) && !(work & PC7_MASK)) {
		ev_add(EV_PC7_DOWN);
	    }
	}
    }
    prev_scan = work;
}

/*---------------------------------------------------------*/
/* 1000HzC^[o^C} (CMT0)                         */
/*---------------------------------------------------------*/

void Isr_CMI0 (void)
{
	static int div10, b;


	*CMCSR0 &= ~0x0080;			/* CMT0荞݃tÕNA */

	Timer++;

	if ((++b % 0x400) == 0)		/* 1024tickLED1𔽓] */
		*PCDR0 ^=  0x0100;

	if (++div10 >= 10) {
		div10 = 0;
		key_timerproc();
	}
}

void delay_ms (					/* msPʂ̃fBC */
	UINT ms
)
{
	for (Timer = 0; Timer < ms; ) ;
}


/*---------------------------------------------------------*/
/* ytF̏                                */
/*---------------------------------------------------------*/

#define SetCCR1(x) ((void(*)(DWORD))((DWORD)SetCCR1_proc + 0x20000000))(x)	/* CCR1ւ̏(LbVGAŎs) */
void SetCCR1_proc (DWORD val)	/* CCR1ւ̏ */
{
	*CCR1; *CCR1;
	*CCR1 = val;
	*CCR1; *CCR1;
}

static
void IoInit (void)
{
	*FRQCR = 0x1104;			/* NbNݒ: I=144M, B=48M, P=24M */

	SetCCR1(0x0000090B);	/* LbVL */

	*PCIOR0 |= 0x0100;		/* PC8:LEDo */
	*PCDR0 |=  0x0100;		/* LED off */

#ifdef USE_SCIF
	scif2_init();			/* SCI2: N81,38400bps */
#endif

	/* CMT0̏(1kHzC^[o^C}) */
	*STBCR7 &= ~0x04;		/* CMTgp */
	*CMCNT0 = 0;			/* JE^NA */
	*CMCOR0 = PCLK / 8 / 1000 - 1;	/* l̐ݒ */
	*CMCSR0 = 0x0040;		/* 荞݋, NbN:PCLK/8 */
	*CMSTR  |= 0x0001;		/* JEgJn */
	*IPR10 = (*IPR10 & 0xFF0F) | 0x0020;	/* 荞݃x:2 */


	*IBNR = 0x4000;				/* WX^oNgp */
	SetImask(0);			/* S荞݋ */

	*PHCR0 = 0;
}

int main (void)
{
	IoInit();				/* ӃW[̏ */

#ifdef USE_SCIF
	xfunc_in = scif2_get;	/* R\[SCIF2Ɍ */
	xfunc_out = scif2_put;
#endif
	if ((*PCPR0 & PC6_MASK) == 0) {
	    slow_slide = 1;
	}

#ifdef USE_SCIF
	xputs("Program started.\n");
#endif
	GRPH_SelectPlane((PIXEL *)GRA1_VRAM_BASE);
	GRPH_Initialize();
	GRPH_Clear(0);
	GCON_SelectPlane((PIXEL *)GRA2_VRAM_BASE);
	GCON_Initialize();
	GCON_cursor_off();

	delay_ms(500);

	/* Flush any pending events */

	while (ev_get(0)) delay_ms(2);

	while (1) {
	    GCON_clear();

	    table_viewer();
	    text_demo();
	}
}
